# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# array.tcl --- Array manipulation functions
#
# Copyright (C) 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require juf::flow
package require Sequence
namespace eval Juf {namespace eval Array {}}

# -----------------------------------------------------------
# PROC: values [SWITCH] [NAME ...]
#
# Returns a list containing the values of all of the elements
# in the arrays specified by the NAME arguments.
# If SWITCH equals to `-unique', a specific value appears
# only once in the list.
# -----------------------------------------------------------

proc Juf::Array::values {args} {
	set v_unique 0
	set v_cont 1
	#
	# Process switches, if any
	#
	while {$v_cont} {
		juf_branch [lindex $args 0] \
				-- {Juf::Sequence::shift args; breaks} \
				-unique {Juf::Sequence::shift args; set v_unique 1} \
				default {set v_cont 0}
	}
	set v_list ""
	foreach name $args {
		upvar $name v_name
		#
		# Build list with the help of an auxiliary array
		#
		foreach v_el [array names v_name] {
			set v_val $v_name($v_el)
			if {![info exists v_arr($v_val)] || !$v_unique} {
				lappend v_list $v_val
			} 
			#
			# Create array element for this value
			#
			set v_arr($v_val) ""
		}
	}
	set v_list
}

# --------------------------------------------
# PROC: sort NAME
#
# Returns list of element names of array NAME,
# sorted according to the element values
# --------------------------------------------

proc Juf::Array::sort {name} {
	upvar $name parr
	#
	# build auxiliary array
	#
	foreach key [array names parr] {
		lappend v_hash($parr($key)) $key
	}
	set v_list ""
	#
	# build result list
	#
	foreach key [lsort [array names v_hash]] {
		set v_keys $v_hash($key)
		foreach v_key $v_keys {
			lappend v_list $v_key
		}
	}
	return $v_list
}
 
package provide Array 0.0.4

# SOME SETTINGS FOR GOOD OLD EMACS:
# Local Variables:
# tcl-default-application: "tclsh"
# End:
