# This is a -*- Tcl -*- library file and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# getopt.tcl --- options processing module
#
# Copyright (C) 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.4
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package require Error
package require Sequence
namespace eval Juf {namespace eval Getopt {}}

# ------------------------------------------------------------------
# PROC: get_options [OPTSPEC ...]
#
# Processes commandline options described by the OPTSPEC arguments.
#
# Each OPTSPEC is a list with an option description and the name
# of a variable for result storage.
#
# Each option description provides one or more option.
# They are distinguished
# by the pipe sign. An option is prefixed with one dash if a
# one-character-string, two dashes otherwise.
#
# Example:
# help|h produces the options -h and --help.
#
# Returns 1 if successful, 0 otherwise.
# ------------------------------------------------------------------

proc Juf::Getopt::get_options {args} {
	global argv

	foreach optspec $args {
		Juf::Sequence::assign $optspec optdescr varname
		#
		# initialize variable
		#
		upvar $varname tmp
		set tmp 0
		foreach optname [split $optdescr |] {
			if {[string length $optname] == 1} {
				set optarr(-$optname) $varname
			} else {
				set optarr(--$optname) $varname
			}
		}
	}
	set tmpargv $argv
	set argv ""

	while {[llength $tmpargv]} {
		set arg [lindex $tmpargv 0]
		switch -glob -- $arg -- {
			# end of options
			eval lappend argv $tmpargv
			set tmpargv ""
		} -?* {
			# valid option ?
			if [info exists optarr($arg)] {
				upvar $optarr($arg) tmp
				set tmp 1
			} else {
				Juf::Error::error "invalid option \"$arg\""
				return 0
			}
		} default {
			lappend argv $arg
		}
		set tmpargv [lrange $tmpargv 1 end]
	} 

	return 1
}

package provide juf::getopt 0.0.4