# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# misc.tcl --- Miscellaneous Functions
#
# Copyright (C) 1996, 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.1
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

# -----------------------------------------------------
# juf_map FUNCTION LIST
#
# Applies FUNCTION to each element of LIST in turn, and
# returns a list of the results.
# -----------------------------------------------------

proc juf_map {function list {name ""}} {
	set result ""
	foreach el $list {
		if {$name != ""} {
			set $name $el
			lappend result [eval $function]
		} else {
			lappend result [eval $function {$el}]
		}
	}
	set result
}

# ---------------------------------------------------------
# juf_misc_scriptname - returns name of current Tcl script
# ---------------------------------------------------------

proc juf_misc_scriptname {} {
	file rootname [file tail [info script]]
}

# --------------------------------------------------------
# juf_misc_shell - prompts for user input and evaluates it
#                  in the current interpreter until user
#				   exits the shell
#
# in:	prompt	user prompt (default "% ")
# --------------------------------------------------------

proc juf_misc_shell {{prompt "% "}} {
	puts -nonewline stdout $prompt
	flush stdout

	while {[gets stdin input] != -1} {
		catch "uplevel $input" result
		puts stdout $result
		puts -nonewline stdout $prompt
		flush stdout
	}
}

# ---------------------------------------------------------------
# juf_log_versioninfo - log version information on stdout
#
# If a script file is currently being evaluated, this function
# looks for the variables `version', `spec' and `pkginfo' with
# a prefix dependent on the script file name in the global
# context and creates version information from the values of
# this variables. At least the variable `version' must exist.
# E.g., if you running the script `~/foo', the variables 
# `foo_version' (script version), `foo_spec' (script description)
# and `foo_pkginfo' (info about package the script belongs to)
# will be looked up.
# ---------------------------------------------------------------

proc juf_log_versioninfo { {myname ""} {chlid stdout} } {
	set info 0

	if {$myname == ""} {
		set myname [file rootname [file tail [info script]]]
	}
	set outbuf ""

	if [uplevel #0 info exists [set myname]_spec] {
		upvar #0 [set myname]_spec buf
		set outbuf "This is $buf version "
	}
	if [uplevel #0 info exists [set myname]_version] {
		upvar #0 [set myname]_version buf
		append outbuf $buf
		set info 1
	}
	if [uplevel #0 info exists [set myname]_pkginfo] {
		upvar #0 [set myname]_pkginfo buf
		append outbuf ", from $buf"
	}
	if {$info} {
		puts $chlid "$outbuf."
	}
}

# ---------------------------------------------------------
# PROC: juf_clock_number TIME FORMAT
#
# Converts aspect of TIME (as retrieved by `clock seconds')
# to a number. FORMAT determines aspect.
# ---------------------------------------------------------

proc juf_clock_number {time format} {
	set v_buf [string trimleft [clock format $time -format $format] 0]
	if [string length $v_buf] {
		return $v_buf
	}
	return 0
}

package provide juf::misc 0.0.1

# SOME SETTINGS FOR GOOD OLD EMACS:
# Local Variables:
# tcl-default-application: "tclsh"
# End: