# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# tpl.tcl --- Template processing functions
#
# Copyright (C) 1997, 1998 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.3
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

proc juf_tpl_find_macro {string start end} {
	if [regexp -indices $start $string v_dummy] {
		set v_pos1 [lindex $v_dummy 0]
		set v_pos2 [lindex $v_dummy 1]
	} else {
		return ""
	}
	set string [string range $string [expr $v_pos2 + 1] end]
	if [regexp -indices $end $string v_dummy] {
		set v_pos3 [expr [lindex $v_dummy 0] + $v_pos2 + 1]
		set v_pos4 [expr [lindex $v_dummy 1] + $v_pos2 + 1]
	} else {
		return ""
	}
	list $v_pos1 $v_pos2 $v_pos3 $v_pos4
}

proc juf_tpl_schedule {start end between arrays} {
	#
	# leading question mark indicates deletion of macro
	# if no value found
	#
	if {[string index $between 0] == "?"} {
		set between [string range $between 1 end]
		set v_preserve 0
	} else {
		set v_preserve 1
	}
	#
	# split macro into variable name and replacement instruction
	#
	set v_list [juf_split $between : 2]
	if {[llength $v_list] == 2} {
		set v_key [lindex $v_list 0]
		set v_instr [lindex $v_list 1]
	} else {
		set v_key $between
		set v_instr "%"
	}
	#
	# array value may be a list
	#
	set v_list [juf_split $v_key # 2]
	if {[llength $v_list] == 2} {
		set v_key [lindex $v_list 0]
		set v_index [lindex $v_list 1]
	} else {
		set v_index -1
	}
	foreach v_aname $arrays {
		upvar 2 $v_aname v_arr
		if [info exists v_arr($v_key)] {
			if {$v_index < 0} {
				#
				# replace with array element
				#
				set v_rpl $v_arr($v_key)
			} else {
				#
				# assume array element is a list
				#
				set v_rpl [lindex $v_arr($v_key) $v_index]
			}
			regsub -all % $v_instr $v_rpl v_buf
			return $v_buf
		}
	}
	if $v_preserve {return "$start$between$end"}
}

proc juf_tpl_run {string {arrays ""} {start "@"} {end "@"}} {
	set v_pos 0
	set v_rmd $string
	
	set v_res [juf_tpl_find_macro $v_rmd $start $end]
	while {$v_res != ""} {
		#
		# first add stuff before match to result
		#
		append v_buf [string range $v_rmd 0 [expr [lindex $v_res 0] - 1]]
		#
		# pass macro to scheduler
		#
		append v_buf [juf_tpl_schedule \
				[string range $v_rmd [lindex $v_res 0] [lindex $v_res 1]] \
				[string range $v_rmd [lindex $v_res 2] [lindex $v_res 3]]\
				[string range $v_rmd [expr [lindex $v_res 1] + 1] \
				[expr [lindex $v_res 2] - 1]] \
				$arrays]
		#
		# create string for next macro search
		#
		set v_rmd [string range $v_rmd [expr [lindex $v_res 3] + 1] end]
		set v_res [juf_tpl_find_macro $v_rmd $start $end]
	}
	append v_buf $v_rmd
}

package provide Template 0.0.3


