# This is a Tcl library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# urltools.itcl --- URL manipulating functions
#
# Copyright (C) 1997 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0.0
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.

# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

# -----------------------------------------------
# PROC: jufwww_getproto URL
#
# Returns protocol part of URL or an empty string
# -----------------------------------------------

proc jufwww_getproto {url} {
	if [regexp -- {^([^/]+):} $url dummy v_proto] {
		return $v_proto
	}
}

# -----------------------------------------------
# PROC: jufwww_gethost URL
#
# Returns host part of URL or an empty string
# -----------------------------------------------

proc jufwww_gethost {url} {
	if [regexp -- {^[^/]*//([^/]+)} $url dummy v_host] {
		return $v_host
	}
}

# -------------------------------------------
# PROC: jufwww_getpath URL
#
# Returns path part of URL or an empty string
# -------------------------------------------

proc jufwww_getpath {url} {
	if [regexp -- {^[^/]*//[^/]+((/[^/]+)+/)} $url dummy v_host] {
		return $v_host
	}
}

# ---------------------------------------------------------------
# PROC: jufwww_getfilename URL
#
# Returns filename part of URL (includes path) or an empty string
# ---------------------------------------------------------------

proc jufwww_getfilename {url} {
	if [regexp -- {^[^/]*//[^/]+(.*)} $url dummy v_name] {
		return $v_name
	}
}

# -------------------------------------------------
# PROC: jufwww_construct RELURL [REFURL]
#
# Constructs absolute URL. RELURL is a relative or
# absolute URL, REFURL is a absolute URL, if given.
# -------------------------------------------------

proc jufwww_construct {relurl {refurl ""}} {
	set v_relprot [jufwww_getproto $relurl]
	#
	# some special protocols lacking relative URL's
	#
	if {![string compare $v_relprot mailto] || \
			![string compare $v_relprot news] } {
		return $relurl
	}
	set v_relhost [jufwww_gethost $relurl]
	if {$v_relprot != "" && $v_relhost != ""} {return $relurl}
	if {$refurl == ""} {
		if {$v_relprot == ""} {
			#
			# relative URL, no reference URL => local file
			#
			return "file://localhost[file join [pwd] $relurl]"
		} else {
			#
			# relative URL, no reference URL => localhost
			#
			if {[string index $relurl 0] != "/"} {
				return "$v_relprot//localhost/$relurl"
			} else {
				return "$v_relprot//localhost$relurl"
			}
		}
	}
	if {$v_relhost != ""} {
		#
		# relative URL with host, reference URL
		#
		return "[jufwww_getproto $refurl]:$relurl"
	} else {
		#
		# relative URL without host, reference URL
		#
		set v_refprot [jufwww_getproto $refurl]
		set v_refhost [jufwww_gethost $refurl]
		set v_refpath [jufwww_getpath $refurl]

		if {$v_refpath == ""} {set v_refpath "/"}
		
		if {[string index $relurl 0] != "/"} {
			return "$v_refprot://$v_refhost$v_refpath$relurl"
		} else {
			return "$v_refprot://$v_refhost$relurl"
		}
	}
}

# SOME SETTINGS FOR GOOD OLD EMACS:
# Local Variables:
# mode: tcl
# tcl-default-application: "tclsh"
# End:
